/*
   ####################################################################################
   #                                                                                  #
   #                             LibCodeBananas V3.0.0                                #
   #                      Dspbanana - Some helpful dsp tools                          #
   #                                                                                  #
   #    Copyright (C) 2006-2010 Philipp Fabian Benedikt Maier (aka. Dexter)           #
   #                                                                                  #
   #    This program is free software; you can redistribute it and/or modify          #
   #    it under the terms of the GNU General Public License as published by          #
   #    the Free Software Foundation; either version 2 of the License, or             #
   #    (at your option) any later version.                                           #
   #                                                                                  #
   #    This program is distributed in the hope that it will be useful,               #
   #    but WITHOUT ANY WARRANTY; without even the implied warranty of                #
   #    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
   #    GNU General Public License for more details.                                  #
   #                                                                                  #
   #    You should have received a copy of the GNU General Public License             #
   #    along with this program; if not, write to the Free Software                   #
   #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA    #
   #                                                                                  #
   #################################################################################### */


/* ## HEADER ########################################################################## */
#ifndef dspbanana_h
#define dspbanana_h

#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN
#endif/*__cplusplus*/

#define PI 3.1415

/* File handling */
EXTERN int getLe32fSample(double *sample, FILE *file);		/* Get a sample from input file */
EXTERN int putLe32fSample(double sample, FILE *file);		/* Put a sample to output file */
EXTERN int saveLe32fSampleFile(double *buffer, int bufferLength, FILE *file);	/* Save sample buffer to an le32f output file */
EXTERN int getGnuplotSample(double *sample, FILE *file);	/* Get a sample from le32f input file */
EXTERN int putGnuplotSample(double sample, FILE *file);		/* Put a sample to an gnuplot file */
EXTERN int saveGnuplotSampleFile(double *buffer, int bufferLength, FILE *file);	/* Save sample buffer to an gnuplot output file */
EXTERN int getU16Sample(double *sample, FILE *file);		/* Get a sample from u16 (sox) input file */
EXTERN int putU16Sample(double sample, FILE *file);		/* Put a sample to u16 (sox) output file */

/* Math toolbox */
EXTERN double complexAbs(double re, double im);			/* Calculate the absolute value from a complex number */
EXTERN double complexArg(double re, double im);			/* Calculate the argument from a complex number */
EXTERN double calcTsample(int samplerate);			/* Calculate the time distance between to samples, also known as T */
EXTERN double frequency2CycleDuration(double frequency);	/* Calculate the cycle duration from a given frequency (T=1/F) */
EXTERN double cycleDuration2Frequency(double cycleDuration);	/* Calculate the frequency from a given cycle duration (F=1/T) */
EXTERN int baud2samples(int baudrate, int samplerate);		/* Calculate the number of samples for one ETU on a serial transmission */
EXTERN int samples2baud(int samplecount, int samplerate);	/* Calculate the baudrate of a serial transmission from the sample count of one ETU */

/* Stream operations */
EXTERN double streamAverage(double sample, double *window, int windowLength, int *state);	/* Calculate a sliding average */
EXTERN double streamMedian(double sample, double *window, int windowLength, int *state);	/* Calculate a sliding median */
EXTERN double streamHighest(double sample, double *window, int windowLength, int *state);	/* Report the highest value of some recend values */	
EXTERN double streamLowest(double sample, double *window, int windowLength, int *state);	/* Report the lowest value of some recend values */	
EXTERN int streamComparator(double sample, double threshold);					/* Convert a float Value to bool by comperator */
EXTERN void streamHistoryPut(double sample, double *history, int historyLength, int *state);	/* Put value to history */
EXTERN double streamHistoryGet(int age, double *history, int historyLength, int *state);	/* Get value from history */
EXTERN int streamDetectSilence(double sample, double levelTreshold, int timeTreshold, int *state);	/* Detect silence in a stream */
EXTERN double streamOscillator(double f, double amp, double phi, double dc, int samplerate, int *status);/* Numerical controlled streamOscillator (f[hz], phi[multiple of PI]) */
EXTERN double streamMixer(double signal1, double signal2);					/* Mixer */
EXTERN double streamAdd(double signal1, double signal2);					/* Add */
EXTERN double streamSubtract(double signal1, double signal2);					/* Subtract (signal1 - signal2) */
EXTERN double streamNoise(double level);							/* Noise generator */
EXTERN double streamRectifier(double signal);							/* Rectifier */
EXTERN int streamSpaceBetweenZeroCrossings(double sample, double zero, int *status);		/* Determine the space between two zero crossings of a signal (Caution: status[3])*/
EXTERN int streamAsyncSerialDecode(char *byte, int sample, int etu, int size, int parity, int stopbits, int debug, int *status); /* Decode a serial bitstream (returncodes: ParityError:-1, Done:0, Busy:1, Caution: status[8]) */

/* Block operations (single block) */
EXTERN double blockAverage(double *data, int length);				/* Calculate average from a given array */
EXTERN double blockMedian(double *data, int length);				/* Calculate median from a given array */
EXTERN int blockHighestPos(double *data, int length);				/* Find the position of the highest value in a given array */
EXTERN int blockHighestCharPos(char *data, int length);				/* Find the position of the highest value in a given char array */
EXTERN int blockHighestIntPos(int *data, int length);				/* Find the position of the highest value in a given int array */
EXTERN int blockHighestLongIntPos(long int *data, int length);			/* Find the position of the highest value in a given long int array */
EXTERN int blockLowestPos(double *data, int length);				/* Find the position of the lowest value in a given array */
EXTERN int blockLowestCharPos(char *data, int length);				/* Find the position of the the lowest value in a given char array */
EXTERN int blockLowestIntPos(char *data, int length);				/* Find the position of the the lowest value in a given int array */
EXTERN int blockLowestLongIntPos(char *data, int length);			/* Find the position of the the lowest value in a given long int array */
EXTERN double blockHighest(double *data, int length);				/* Find the highest value in a given array */
EXTERN char blockHighestChar(char *data, int length);				/* Find the highest value in a given char array */
EXTERN int blockHighestInt(int *data, int length);				/* Find the highest value in a given int array */
EXTERN long int blockHighestLongInt(long int *data, int length);		/* Find the highest value in a given long int array */
EXTERN double blockLowest(double *data, int length);				/* Find the lowest value in a given array */
EXTERN char blockLowestChar(char *data, int length);				/* Find the lowest value in a given char array */
EXTERN int blockLowestInt(int *data, int length);				/* Find the lowest value in a given int array */
EXTERN long int blockLowestLongInt(long int *data, int length);			/* Find the lowest value in a given long int array */
EXTERN void blockComparator(double *data, int length, double threshold);	/* Calculate a comperator behaviour for a given array */
EXTERN void blockBinarySpikeRemoval(char *data, int length, int minlength);	/* Remove spikes from a binary block (only ones and zeros!) */
EXTERN void blockAdd(double *data, int length, double summand);			/* Add a random but fixed value to a block */
EXTERN void blockSubtract(double *data, int length, double subtrahend);		/* Subtract a random but fixed value from a block */
EXTERN void blockGain(double *data, int length, double gain);			/* Gain a block (multiply with a random but fixt value) */
EXTERN void blockOscillator(double *data, int length, double f, double amp, double phi, double dc, int samplerate);	/* Numerical controlled streamOscillator (f[hz], phi[multiple of PI]) */
EXTERN void blockRectifier(double *data, int length);				/* Rectifier */
EXTERN void blockNoise(double *data, int length, double level);			/* Noise Generator */

/* Blocks operations (multiple blocks) */
EXTERN void blocksMixer(double *data1, double *data2, double *dataOut, int length);	/* Mixer (data1 * data2) */
EXTERN void blocksAdd(double *data1, double *data2, double *dataOut, int length);	/* Add (data1 + data2) */
EXTERN void blocksSubtract(double *data1, double *data2, double *dataOut, int length);	/* Subtract (data1 - data2) */
EXTERN void blocksDivide(double *data1, double *data2, double *dataOut, int length);	/* Divide (data1 / data2) */

/* Cross Corellation functions */
EXTERN double crossCorrelate(double *data1, double *data2, int length1, int length2, int tau);				/* Cross correlate two double vectors once at the position tau */
EXTERN int blockCrossCorrelate(double *data1, double *data2, double *result, int length1, int length2);			/* Cross correlate two double vectors at all possible tau positions */
EXTERN char charCrossCorrelate(char *data1, char *data2, int length1, int length2, int tau);				/* Cross correlate two char vectors once at the position tau */
EXTERN int blockCharCrossCorrelate(char *data1, char *data2, char *result, int length1, int length2);			/* Cross correlate two char vectors at all possible tau positions */
EXTERN int intCrossCorrelate(int *data1, int *data2, int length1, int length2, int tau);				/* Cross correlate two int vectors once at the position tau */
EXTERN int blockIntCrossCorrelate(int *data1, int *data2, int *result, int length1, int length2);			/* Cross correlate two int vectors at all possible tau positions */
EXTERN long int longIntCrossCorrelate(long int *data1, long int *data2, int length1, int length2, int tau);		/* Cross correlate two long int vectors once at the position tau */
EXTERN int blockLongIntCrossCorrelate(long int *data1, long int *data2, long int *result, int length1, int length2);	/* Cross correlate two long int vectors at all possible tau positions */
EXTERN int binaryCrossCorrelate(char *data1, char *data2, int length1, int length2, int tau);				/* Cross correlate two binary vectors once at the position tau */
EXTERN int blockBinaryCrossCorrelate(char *data1, char *data2, char *result, int length1, int length2);			/* Cross correlate two binary vectors at all possible tau positions */

/* Manchester encoding/decoding */
#define G_E_THOMAS 1
#define IEEE_802_3 0
EXTERN int manchesterEncode(char *dataIn, char *dataOut, int dataLength, int mode);	/* Manchester-Encode binary integer array (mode=0 ieee802.3, mode=1 G.E.Thomas) */
EXTERN int manchesterDecode(char *dataIn, char *dataOut, int dataLength, int mode);	/* Manchester-Decode binary integer array (mode=0 ieee802.3, mode=1 G.E.Thomas) */
EXTERN int manchesterCheck(char *data, int dataLength);					/* Check array content for manchester encodeing rules violation */
EXTERN int manchesterLength(char *data, int dataLength);				/* Find last valid manchester symbol (transmission length) */
EXTERN int manchesterAlign(char *dataIn, char *dataOut, int dataLength, char *preamble, int preambleLength);	/* Align manchester data  */

/* Shift operations */
EXTERN void shiftDoubleBufferRight(double *data, int length, double input, double *output);		/* Shift double buffer right */
EXTERN void shiftDoubleBufferLeft(double *data, int length, double input, double *output);		/* Shift double buffer left */
EXTERN void shiftIntBufferRight(int *data, int length, int input, int *output);				/* Shift int buffer right */
EXTERN void shiftIntBufferLeft(int *data, int length, int input, int *output);				/* Shift int buffer left */
EXTERN void shiftLongIntBufferRight(long int *data, int length, long int input, long int *output);	/* Shift long int buffer right */
EXTERN void shiftLongIntBufferLeft(long int *data, int length, long int input, long int *output);	/* Shift long int buffer left */

/* Casting */
EXTERN void double2char(double *doubleData, char *charData, int length);	/* Convert double array to char */
EXTERN void int2char(int *intData, char *charData, int length);			/* Convert int array to char */
EXTERN void longInt2char(long int *intData, char *charData, int length);	/* Convert long int array to char */

/* Demodulators */
EXTERN int demod2fsk(double sample, double centerFrequency, double zeroOffset, int samplerate, int debug, int *status);	/* Demodulate a 2fsk signal (Caution: status[3]) */

/* Typo correction */
#define blockComperator(data, length, threshold)	blockComparator(data, length, threshold)
#define streamComperator(sample,threshold)		streamComparator(sample, threshold)

#endif/*dspbanana_h*/
/* #################################################################################### */
 







