/*
   ####################################################################################
   #                                                                                  #
   #                             LibCodeBananas V3.0.0                                #
   #                     Confbanana Configuration File Parser                         #
   #                                                                                  #
   #    Copyright (C) 2006-2010 Philipp Fabian Benedikt Maier (aka. Dexter)           #
   #                                                                                  #
   #    This program is free software; you can redistribute it and/or modify          #
   #    it under the terms of the GNU General Public License as published by          #
   #    the Free Software Foundation; either version 2 of the License, or             #
   #    (at your option) any later version.                                           #
   #                                                                                  #
   #    This program is distributed in the hope that it will be useful,               #
   #    but WITHOUT ANY WARRANTY; without even the implied warranty of                #
   #    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
   #    GNU General Public License for more details.                                  #
   #                                                                                  #
   #    You should have received a copy of the GNU General Public License             #
   #    along with this program; if not, write to the Free Software                   #
   #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA    #
   #                                                                                  #
   #################################################################################### */

/* ## HEADER ########################################################################## */
#ifndef toolbanana_h
#define toolbanana_h

#include <stdbool.h>								/* Include Bolean types */

#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN
#endif/*__cplusplus*/

/* Produce hexlisting of a string */
EXTERN void printfHexlistingLen(char *dataString, int width, int length);	/* Produce Hexdump with fixed length */
EXTERN void printfHexlisting(char *dataString, int width);			/* Produce Hexdump */
EXTERN void printfHex(char *data, int length, char seperator);			/* Print buffer content in hexadecimal */
EXTERN void sprintfHex(char *string, char *data, int length, char seperator);	/* Print buffer content in hexadecimal to a given buffer */
EXTERN void hexBinAsciiDump(char *data, int length);				/* Display a full detailed hex-binaray-ascii data dump */
EXTERN void hexBinAsciiDumpToString(char *string, char *data, int length);	/* Generate a string containing a full detailed hex-binaray-ascii data dump */
EXTERN void hexBinAsciiDumpLive(int data, int reset);				/* Display a full detailed hex-binaray-ascii data dump in realtime (call with reset=1 to reset) */

/* Print the binary complement of numbers */
EXTERN void printfBinary(int data);						/* Print integer in binary (8-bit) */
EXTERN void sprintfBinary(char *string, int data);				/* Print integer in binary (8-bit) to a string */
EXTERN void printfBinary16(int data);						/* Print integer in binary (16-bit) */
EXTERN void sprintfBinary16(char *string, int data);				/* Print integer in binary (16-bit) to a string */
EXTERN void printfBinary32(int data);						/* Print integer in binary (32-bit) */
EXTERN void sprintfBinary32(char *string, int data);				/* Print integer in binary (32-bit) to a string */
EXTERN void printfBinaryArray(char *samples, int samplesLength, int width);	/* Dump a binary integer array as a formatted array of 1 and 0 on the screen */

/* String processing */
EXTERN int memcmpMasked(char *data1, char *data2, char *mask, int length);	/* Compare memory under application of a mask */
EXTERN char *strtrim(char *string);						/* Remove leading and following spaces */
EXTERN void ltrimStringChar(char *string, char char2trim);			/* Remove leading characters that match char2trim */
EXTERN void rtrimStringChar(char *string, char char2trim);			/* Remove following characters that match char2trim */
EXTERN void trimStringChar(char *string, char char2trim);			/* Remove leading and following characters that match char2trim */
EXTERN void trimString(char *string);						/* Remove leading and following spaces (improved version of strtrim()) */
EXTERN int strCutOut(char *left, char *right, char *string);			/* Cut out the data between left and right delimeter */
EXTERN int strReplace(char *old, char *new, char *string, unsigned int buffersize);	/* Replace old with new in string (-1 string not found, -2 buffer too small) */
EXTERN int strReplaceAll(char *old, char *new, char *string, unsigned int buffersize);	/* Replace all old with new in string (-1 string not found, -2 buffer too small) */
EXTERN void listExtract(char *list, int listLength, char delimeter, char *item, int itemLength, int itemNumber);	/* Extract item from delimeter seperated list */
EXTERN void stripChar(char *string, char char2strip, int length);		/* Strip a defined character out of a string */
EXTERN void replaceChar(char *string, char char2replace, char replacementChar, int length);	/* Replace a defined character in a string */
EXTERN void replaceCharRange(char *string, char from, char to, char replacementChar, int length);	/* Replace a defined character range in a string */
EXTERN int countChar(char *buffer, char char2Count, int length);		/* Count the appearing of a specific character in a data string */
EXTERN int bufferEmpty(char *buffer, int bufferLen);				/* Check if buffer is empty (contains only zeros) */
EXTERN int removeLeadingBytes(char *data, int length, char byte);		/* Remove leading bytes of a buffer (e.g. 0x55, 0x55, 0x23, 0x42 ... becomes 0x23, 0x42 ...) */
EXTERN int removeLeadingBytesCount(char *data, int length, int count);		/* Remove a fixed number of leading bytes of a buffer */
EXTERN void ucase(char *string);						/* Convert all letters in the string to upper case */
EXTERN void lcase(char *string);						/* Convert all letters in the string to lower case */

/* Bit manipulation / transformations */
EXTERN int setBit(int byte, int bitId, bool status);				/* Write a single bit */
EXTERN bool getBit(int byte, int bitId);					/* Read a single bit */
EXTERN bool isHex(char data);							/* Check if a character represents a hexadecimal digit (0-9 or A-F) */
EXTERN bool isPrintable(char data);						/* Check if a character represents a printable digit */
EXTERN int hex2int4(char *hexData);						/* Convert character hex-nibble into integer number */
EXTERN int hex2int8(char *hexData);						/* Convert character string hex-nibble into 8-bit integer number */
EXTERN int hexString2numeric(char *string, char *numbers, int numbersLength);	/* Convert a string with hexadecimal characters to its numberic counterpart */
EXTERN void numeric2hexString(char *numbers, int numbersLength, char *string, char *delimeter);	/* Convert a an arry with numeric data to a string with hexadecimal characters */
EXTERN int countBits(char byte);						/* Find out how many bits in a byte are set */
EXTERN int getBitPos(int integer);						/* Get the position of the first bit that is set to 1 */
EXTERN unsigned char swapBitOrder(unsigned char byte);				/* Swap the bit order in a byte (MSB becomes LSB and so on) */
EXTERN unsigned int swapBitOrder16(unsigned int word);				/* Swap the bit order in a 16-bit-word (MSB becomes LSB and so on) */
EXTERN char swapNibbles(char byte);						/* Swap the two nibbles of a byte */
EXTERN void swapNibblesBuffer(char *buffer, int length);			/* Swap the two nibbles of all bytes in a given buffer */
EXTERN void flipBuffer(char *data, int length);					/* Flip buffer content (0x01, 0x02, 0x03 becomes 0x03, 0x02, 0x01 ) */
EXTERN void invertBits(char *input, char *output, int length);			/* Invert bits in an unpacked binary buffer e.g. 0x00000001 becomes 0x00000001 and vice versa */
EXTERN int packBits(char *binaryArray, int length, char *packedData);		/* Pack a binary char array */
EXTERN int ascii2BinaryBits(char *string, char *binary, int binaryMaxLength);	/* Convert Ascii string (e.g. "100111010") into an unpacked binary array */

/* CRC calculation */
EXTERN int crcComputationCycle(char data, int polynom, int crc);		/* Compute the CRC for a single byte in a stream */
EXTERN int computeCrc(char *data, int length, int polynom, int initialValue);	/* Compute the CRC of a given buffer */
EXTERN int computeCrcPolynommask(int *polynom, int length, int type);		/* Compute the Polynom constant from given exponents (e.g.: x^16 + x^15 + x^2 + x^0 ==> int p[] = {16,15,2,0}; */

/* File IO */
EXTERN int loadFile(char *buffer, int length, char *filename);			/* Write a buffer to file */
EXTERN int storeFile(char *buffer, int length, char *filename);			/* Load a file to buffer */
EXTERN int testFile(char *filename);						/* Test if a file is ready to get opend */
EXTERN int testReadonlyFile(char *filename);					/* Test if a file is ready to get opend readonly */
EXTERN void newFileEnding(char *filename, char *newFilename, char *newEnding);	/* Generate a new filename with a new .ending */
EXTERN int findFile(char *filename, char *foundFilePath);			/* Try to find a file in current directory, home directory or /etc directory */

/* User interface */
EXTERN bool userYesNo(char *question);						/* Ask the user a Yes/No Question */
EXTERN char userMenueButton(char *validButtons, char *prompt, char *errormsg);	/* Filter user input by a list of allowed menue buttens */
EXTERN int scanfHexArray(char *data);						/* Read a an array of hex-input numbers to an array */
EXTERN int scanfBinaryArray(char *data);					/* Read a an array of binary numbers to an array */
EXTERN void editBuffer(char *data, int length);					/* Edit character array */
EXTERN void editTextLine(char *string, int length);				/* Edit a line of text */
EXTERN void drawBargraph(int value, int max, char style);			/* Draw a bargraph on the screen */
EXTERN void drawGraph(char *data, int length, int highest, int height, int xPos, int yPos, char pen, char paper, int penWidth); /* Visualize array content using a graph */
EXTERN int getScreenHeight(void);						/* Get screen height ($LINES) */
EXTERN int getScreenWidth(void);						/* Get screen width ($COLUMNS) */
EXTERN char getch(void);							/* Read a single character from keyboard */
EXTERN int kbhit();								/* Check if the keyboard is hit / return the number of bytes in buffer */
EXTERN void echoOff(void);							/* Turn console echo off */
EXTERN void echoOn(void);							/* Turn console echo on */

/* Shift operations */
EXTERN void shiftBufferRight(char *data, int length, char input, char *output);	/* Shift buffer right */
EXTERN void shiftBufferLeft(char *data, int length, char input, char *output);	/* Shift buffer left */

/* Scanf replacements */
EXTERN int recallingScanf(char *formatString, void *target, int id);		/* a scanf that can remember previous inputs (one pointer only) */
EXTERN void setRecallingScanfHistory(char *filename);				/* Set user specific filename for the recalling scanf history */
EXTERN int improvedScanf(char *formatString, void *target);			/* improved scanf (accepts crlf only inputs, one pointer only) */
EXTERN int foolproofScanf(char *formatString, void *target, char *errormsg);	/* improved scanf (accepts only correct inputs, one pointer only) */

/* Ansi control codes: */
#define clearScreen()		printf("\033[2J")		/* Cĺear the screen */
#define clearLine()		printf("\033[K")		/* Clear the courrent line */

#define setInvisibleText()	printf("\033[8m")		/* Make the text invisible */
#define setInverseText()	printf("\033[7m")		/* Invert text */
#define setBlinkingText()	printf("\033[5m")		/* Make text blinking */
#define setUnderlineText()	printf("\033[4m")		/* Underline text */
#define setFatText()		printf("\033[1m")		/* Make text bold */
#define setScreenColor(color)	printf("\033[%im",color)	/* Set the screen color (30..47) */
#define resetScreenSettings()	printf("\033[0m")		/* Make screen normal again */

#define storeCursorPosition()	printf("\033[s")		/* Store cursor position (in the Terminal!) */
#define recallCursorPosition()	printf("\033[u")		/* Recall cursor position */
#define fetchCursorPosition()	printf("\033[6n")		/* Fetch cursor position (grab it with getchar()) */

#define setCursorPosition(x, y)	printf("\033[%i;%iH",y,x)	/* Set absolute cursor position (use with care!) */
#define setCursorHome()		printf("\033[H")		/* Set cursor to home */

#define setCursorLeft(n)	printf("\033[%iD",n)		/* make cursor going n digits left */
#define setCursorRight(n)	printf("\033[%iC",n)		/* make cursor going n digits right */
#define setCursorUp(n)		printf("\033[%iA",n)		/* make cursor going n digits up */
#define setCursorDown(n)	printf("\033[%iB",n)		/* make cursor going n digits down */

EXTERN void lineAid(void);	/* Print a lineal to find out where the 80 char border exactly is */

#endif/*toolbanana_h*/
/* #################################################################################### */
